--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2019 Adobe
All Rights Reserved.

NOTICE: All information contained herein is, and remains
the property of Adobe and its suppliers, if any. The intellectual
and technical concepts contained herein are proprietary to Adobe
and its suppliers and are protected by all applicable intellectual
property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material
is strictly forbidden unless prior written permission is obtained
from Adobe.


----------------------------------------------------------------------------]]--



--==============================================================================

local LrAssetUtils = {}

local function isLuaAvailable( name )
  if package.loaded[ name ] then
    return true
  else
    for _, searcher in ipairs( package.searchers or package.loaders ) do
      local loader = searcher( name )
      if type( loader ) == 'function' then
        package.preload[ name ] = loader
        return true
      end
    end
    return false
  end
end

local function optRequire( name ) 
	if isLuaAvailable( name ) then
	 	local result = require( name )
		return result
	else 
		return nil
	end
end


local LrmAssetUtils = optRequire( 'LrmAssetUtils' )
local LriAssetUtils = optRequire( 'LriAssetUtils' )
local WFCatFileSpec = optRequire( 'WFCatFileSpec' )

--------------------------------------------------------------------------------

function LrAssetUtils.hasProxyLocally( docValue )
	
	-- SPAM [matthewj 2019-04-08]: Ideally these would all get consolidated to a single implementation,
	-- but currently each client has a slightly different _localOnly path to the binary files right now
	local hasProxyLocally = ( LrmAssetUtils and LrmAssetUtils.hasProxyLocally( docValue ) ) -- android
					or ( LriAssetUtils and LriAssetUtils.hasProxyLocally( docValue ) ) -- ios
					or ( WFCatFileSpec and not not WFCatFileSpec.getProxyRelativePathFromDocValue( docValue ) ) -- desktop
	
	return hasProxyLocally
	
end

--------------------------------------------------------------------------------

function LrAssetUtils.hasMasterLocally( docValue )
	
	-- SPAM [matthewj 2019-04-08]: Ideally these would all get consolidated to a single implementation,
	-- but currently each client has a slightly different _localOnly path to the binary files right now
	local hasMasterLocally =  ( LrmAssetUtils and LrmAssetUtils.hasMasterLocally( docValue ) ) -- android
					or ( LriAssetUtils and LriAssetUtils.hasMasterLocally( docValue ) ) -- ios
					or ( WFCatFileSpec and not not WFCatFileSpec.getOriginalRelativePathFromDocValue( docValue ) ) -- desktop

	return hasMasterLocally
	
end

--------------------------------------------------------------------------------

return LrAssetUtils