--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFPlatformInfo = require "WFPlatformInfo"

local LriCatalogIndexDefinitions = require "LriCatalogIndexDefinitions"

local tablesToReindexByTag = {}

--[[ 
	When the index store map server is initialized for explicit forced updates, 
	we force update a list of index store tables associated with an auto-generated version tag. 
	(Lrm can auto-generate the version tag because forced updates are explicit)

	This is necessary for reindexing progress.
	
	The mapping between version tag and the 
	tables-to-update is stored within tablesToReindexByTag, the format of which is:

	<key>: a version tag
	<value>: a list of tables to update

	Format of version tag: 		
	
		<major>.<minor>.<patch>.<dateEdited>
	
]] --

local appVersion = WFPlatformInfo.getAppVersion() or '0.0.0'
local build = WFPlatformInfo.getBuild() or '0' -- nil for Xcode builds
local adobeProductBuild = WFPlatformInfo.getAdobeProductBuild() or 'unknown' -- git commit rev.
local tag = string.format('%s_%s_%s', appVersion, build, adobeProductBuild )

tablesToReindexByTag[ tag ] = {
	'albums',
	'album_asset',
	'assetProfileLinks',
	'avatarCacheReferences',
	'blocked_uploads',
	'cacheReferences',
	'comment',
	'conflicts',
	'coreInfo',
	'errors',
	'missingBinariesOnOz',
	'presetsProfiles',
	'profileRegistration',
	'quota_exceeded',
	'renditionRevisions',
	'space',
	'space_album',
	'spacefavorite',
	'keywords',
	'persons',
	'faces',
	'feedback_user',
	'space_invite',
	'space_user',
	'trashAssets'
}

LriCatalogIndexDefinitions.tablesToReindexByTag = tablesToReindexByTag

--------------------------------------------------------------------------------

return LriCatalogIndexDefinitions
