--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require "Rx"

	-- Utilities
local KSPathUtils = require "KSPathUtils"
local WFFileUtils = require "WFFileUtils"

	-- WF.oz_client.managed_catalog
local WFCatUser = require 'WFCatUser'
local WFCatSpec = require 'WFCatSpec'
local WFCatalog = require 'WFCatalog'

local WFRxScheduler = require "WFRxScheduler"

	-- WF.oz_client.sync_agent
local OzCatalogUrls = require 'OzCatalogUrls'

local LriForceIndexUpdateModel = {}

--------------------------------------------------------------------------------

function LriForceIndexUpdateModel.makeModel(rootCatalogPath, catalogId, userId)

	-- The result of this query isn't interesting, just the side effect,
	-- that is the population of the tables mentioned in the query.

	-- Note: adding all the tables below to the actual query triggers a sqlite 'hang' bug
	-- for at least the following sqlite3 versions: 3.13.0 - 3.17.0.
	-- Workaround: list all but one of the tables in a sql comment. Good enough
	-- to trigger the indexing.
	local sql = [[
		SELECT date()
		-- albums, album_asset, assetProfileLinks, avatarCacheReferences, blocked_uploads,
		-- cacheReferences, comment, conflicts, coreInfo, errors, missingBinariesOnOz,
		-- presetsProfiles, profileRegistration, quota_exceeded, renditionRevisions, space,
		-- space_album, spacefavorite, keywords, persons, faces, feedback_user, space_invite,
		-- space_user, trashAssets]]

	local userInfoCachePath = KSPathUtils.child( KSPathUtils.getStandardFilePath 'caches', 'Adobe' )
	WFFileUtils.mkdir( userInfoCachePath )
	userInfoCachePath = KSPathUtils.child( userInfoCachePath, 'user' )
	WFFileUtils.mkdir( userInfoCachePath )

	local serverBaseUrl = OzCatalogUrls.baseServerUrl
	local user = WFCatUser( serverBaseUrl, userId, userInfoCachePath )
	local catSpec = WFCatSpec( user, catalogId )
	local catalogOptions = {
		indexSpecScript = 'LriCatalogIndexDefinitionsForceReindex.lua'
	}

	local catalog = WFCatalog( catSpec, rootCatalogPath, catalogOptions )

	local result = catalog:indexStore():queryOnce( {
		sql = sql,
		quiesceUntilUpToDate = true,
	} )
	:flatMap( function( rows )

		local dummy = rows and rows[ 1 ] and rows[ 1 ][ 1 ]

		return catalog:docStore():halt()
			:concat( catalog:indexStore():halt() )
			:concat( Rx.Observable.just( dummy ) )

	end )
	:map( function( value )

		catalog:close()
		user:close()

		return { result = value }
	end )

	local progress = catalog:indexStore():getReindexingProgress()
		:map( function( progress )
			return { progress = progress }
		end )
		:calm( 1, WFRxScheduler.sharedScheduler() )
		:merge( result )
		
	return {
		value  = progress
	}

end

--------------------------------------------------------------------------------

return LriForceIndexUpdateModel
