--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require 'Rx'

	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'

local WFArguments = require 'WFArguments'

--------------------------------------------------------------------------------

local LriLibraryDatabaseModelImp = {}

--==============================================================================

function LriLibraryDatabaseModelImp.docValue( parentModel, docId, docType )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )
	
	if docType == nil then
		return {
			value = docStore:docValue( docId )
		}
	
	else
	
		return {
			value = docStore:docValueOnceP( docType, docId )
		}
		
	end
		
end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.docValueWithFieldSpec( parentModel, docId, optFieldSpec )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )

	return {
		value = docStore:docValue( docId, nil, optFieldSpec )
	}

end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.ongoingDocValue( parentModel, docId, docType )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )
	
	if docType == nil then
	
		return {
			value = docStore:ongoingDocValue( docId )
		}
		
	else
	
		return {
			value = docStore:docValueB( docType, docId )
		}
	
	end
	
	
end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.bulkDocValue( parentModel, assetIds )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )
		
	return {
		value = docStore:bulkDocValue( assetIds )
	}
	
end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.applyOverlay( parentModel, overlay )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )
	
	return {
		value = docStore:applyOverlay( overlay )
	}

end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.queryIndexStore( parentModel, args, ... )

	WFArguments.check( 'table', 'table' )
	
	local catalog = assert( parentModel.catalog )
	local indexStore = assert( catalog:indexStore() )
	
	local n = select( '#', ... )
	
	if n > 0 then
	
		local queryArguments = { ... }
		
		for i = 1, n do 
			table.insert( args, queryArguments[ i ] )
		end
		
	end

	if ( args.oneShot == true ) then

		return {
			value = indexStore:queryOnce( args )
				:map( function( rows )

					return rows

				end )
		}

	else

		return {
			value = indexStore:query( args )
				:map( function( rows )

					return rows

				end )
		}

	end

end

--------------------------------------------------------------------------------


function LriLibraryDatabaseModelImp.makeBulkDocValueModel( parentModel, assetIds, keyPaths )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )

	local fieldSpec = {}

	-- the fieldSpec format is different from the C++ keyPaths format
	-- (which was chosen because it can be written as C++ literals)
	for _, infoPath in ipairs( keyPaths ) do
		KSTableUtils.setNested( fieldSpec, true, unpack( infoPath ) )
	end

	fieldSpec._id = true

	local docValues = docStore:bulkDocValue( assetIds, fieldSpec )
		:map( function( strippedDocValues )

			local result = {}

			for _, docValue in ipairs( strippedDocValues ) do
				result[ docValue._id ] = docValue
			end

			return result

		end )

	return {
		value = docValues
	}

end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.setAssetCustomData( parentModel, assetId, attribute, data, isValidForCurrentRevision )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )

	local localOnly = {}
	local customFields = {}
	local customData = {
		isValidForCurrentRevision = isValidForCurrentRevision,
		currentRevision = '',
		data = data,
	}
	
	customFields[attribute] = customData
	localOnly['lrm_customData'] = customFields
	
	local result = docStore:docValue( assetId )
	:flatMapLatest( function( docValue )

		if ( docValue._empty or docValue._deleted ) then
			return Rx.Observable.just( { state = false, error = 'document empty or deleted' } )
		end

		customData.currentRevision = docValue._revId

		return docStore:applyOverlay( {
			_id = assetId,
			type = 'asset',
			_localOnly = localOnly,
		} )
		:map( function()
			return { state = true }
		end )
		:catch( function( error )
			return Rx.Observable.just( { state = false, error = error } )
		end )

	end )
	:catch( function( error )
		return Rx.Observable.just( { state = false, error = error } )
	end ) 

	return {
		value = result
	}
end

--------------------------------------------------------------------------------

function LriLibraryDatabaseModelImp.compact( parentModel, optVacuum )

	local catalog = assert( parentModel.catalog )
	local docStore = assert( catalog:docStore() )
	
	return {
		value = docStore:compact( optVacuum )
	}

end

--------------------------------------------------------------------------------

return LriLibraryDatabaseModelImp
