--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.utils
local KSPathUtils = require 'KSPathUtils'

	-- WF.core.os
local KSDate = require 'KSDate'
local KSData = require 'KSData'

	-- WF.core.diagnostic_log
local WFDiagnosticLog = require 'WFDiagnosticLog'

	-- WF.oz_client.sync
local WFCatSyncStatus = require 'WFCatSyncStatus'

	-- WF.lrmobile
local LrmConfig = require 'LrmConfig'

--==============================================================================

local LriLibraryStateModelImp = {}

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:applicationStateModel()

	local this = self
	
	return {
		value = Rx.ReplaySubject( 1 ),
		
		events = {
		
			setIsInLoupe = function( self, isInLoupe )
			
				this._isInLoupe:onNext( isInLoupe )
				
			end,
			
			setIsWiFiOnlySync = function( self, isWiFiOnlySync )
			
				this._isWiFiOnlySync:onNext( isWiFiOnlySync )
				
			end,
			
			setReachabilityStatus = function( self, isReachable )
				
				this._isReachable:onNext( isReachable )
				
			end,
			
			setBinaryPreferenceMaster = function( self, useMaster )
			
				local strategy = LrmConfig.StrategyPreferProxy
				
				if useMaster then
					strategy = LrmConfig.StrategyPreferMaster
				end
				
				this._binaryDownloadStrategy:onNext( strategy )
				
			end
		},
	}
	
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:updateAcrProfileDatabaseModel()

	local result = self._updateAcrProfileDatabase
		:map( function( value )
			return {
				updateAcrProfileDatabase = value,
				timestamp = KSDate.timeInIso8601Format( nil, false )
			}
		end )
	
	return {

		value = result,
	}
	
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:hasAlbums()
	
	return {
		valueB = self.hasAlbumsB,
	}
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:catalogAssetCount()

	local result = self.catalog:indexStore():query {
		sql = [[SELECT count(docId) from coreInfo]],
		calm = 20,
	}
	:map( function( data )
	
		local assetCount = 0
		
		if data and data[ 1 ] and data[ 1 ][ 1 ] then
			assetCount = data[ 1 ][ 1 ]
		end
		
		return assetCount
		
	end )
	
	return {
		value = result
	}
	
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:generateDiagnosticLog()

	local result = Rx.ReplaySubject( 1 )
	
	WFDiagnosticLog.generateHtmlReportP( { appName = 'Lr mobile' } )
	:whenResolvedDo( function( status, data )
	
		if status == 'data' then
			local basePath = KSPathUtils.getStandardFilePath( 'home' )
			local p = KSPathUtils.child( basePath, "Library" )
			p = KSPathUtils.child( p, "DiagnosticLog.html" )
			local htmlData = KSData.make( data )
			KSData.saveToUniquelyNamedFile( htmlData, p )
			result:onNext( { state = true, diagnosticData = data } )
		else
			result:onNext( { state = false, error = data } )
		end
	end ):runUntilDone()
	
	return {
		value = result,
	}
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:hasInitialSyncCompletedModel()

	local hasCompletedInitialSync = self._wfCatalogStream
	
		:flatMapLatest( function( wfCatalog )
		
			return WFCatSyncStatus.isDoingInitialSync( wfCatalog )
			
		end )
		
		:map( function( isDoingInitialSync )

			return not isDoingInitialSync
			
		end )
		
	return {
		value = hasCompletedInitialSync
	}
	
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:forceUpgrade()

	local syncClient = self.catalog:getSyncClient()
	
	local result
	
	if syncClient then
		result = syncClient:getRequiredUpdateStream()
	else
		result = Rx.Observable.just( false )
	end		
		
	return {
		value = result
	}
	
end

--------------------------------------------------------------------------------

function LriLibraryStateModelImp:isOzCatalogDeleted()

	local syncClient = self.catalog:getSyncClient()
	
	local result
	
	if syncClient then
		result = syncClient:isOzCatalogDeleted()
	else
		result = Rx.Observable.just( false )
	end		
		
	return {
		value = result
	}
	
end

--------------------------------------------------------------------------------

return LriLibraryStateModelImp
