--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.utils
local WFCapabilities = require 'WFCapabilities'
local KSTableUtils = require 'KSTableUtils'
local WFArguments = require 'WFArguments'

	-- WF.core.os
local WFDeviceInfo = require 'WFDeviceInfo'

	-- WF.oz_client.catalog
local WFOzContentTypes = require 'WFOzContentTypes'
local WFCatOzErrors = require 'WFCatOzErrors'
	-- WF.core.os
local KSPathUtils = require "KSPathUtils"
local WFFileUtils = require "WFFileUtils"
local WFPlatformInfo = require 'WFPlatformInfo'
	-- WF.lrmobile
local LrmConfig = require 'LrmConfig'

--------------------------------------------------------------------------------

local derefOpt = KSTableUtils.derefOpt

--------------------------------------------------------------------------------

LrmConfig.registerBinaryStrategyCheck()

--==============================================================================

local LrmAssetUtils = {}

--------------------------------------------------------------------------------

function LrmAssetUtils.getLocalUrl( docValue )

	local localUrl = derefOpt( docValue, '_localOnly', 'localUrl' )
			
	if not localUrl then
	
		local deviceId = WFDeviceInfo.uniqueDeviceId()
		
		local importSourceDeviceId = derefOpt( docValue, 'importSource', 'uniqueDeviceId' )
		local localAssetId = derefOpt( docValue, 'importSource', 'localAssetId' )
		
		if localAssetId and ( importSourceDeviceId == deviceId ) then
		
			localUrl = localAssetId
			
		end
		
	end
	
	return localUrl
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasProxyOnOz( docValue )

	local proxy_href = docValue
				and docValue._serverProvidedMetadata
				and docValue._serverProvidedMetadata.links
				and docValue._serverProvidedMetadata.links[ "/rels/proxy_type/" ..  WFCapabilities.callFilters( "OzCatalogService.defaultProxyType", 2560 ) ]
				and docValue._serverProvidedMetadata.links[ "/rels/proxy_type/" .. WFCapabilities.callFilters( "OzCatalogService.defaultProxyType", 2560 ) ].href
				
	if proxy_href then
		return true
	end

	return false
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasProxyLocally( docValue )
	
	local localPath = derefOpt( docValue, '_localOnly', 'proxy', 'filePath' )
	if localPath then
		return true
	end
	
	return false
	
end

function LrmAssetUtils.getProxyPath( docValue )
    local proxyPath = derefOpt( docValue, '_localOnly', 'proxy', 'filePath' )

    return proxyPath
end

function LrmAssetUtils.getMasterPath( docValue )

	local masterPath = ""

	if LrmAssetUtils.hasMasterLocally (docValue) then
		masterPath = derefOpt( docValue, '_localOnly', 'originalFilePath' )
		if not masterPath then
			masterPath = derefOpt( docValue, '_localOnly', 'latestOriginal', 'filePath' )
		end
	end

	return masterPath
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasProxy( docValue )

	local hasProxy = LrmAssetUtils.hasProxyOnOz( docValue )
	
	if not hasProxy then
		hasProxy = LrmAssetUtils.hasProxyLocally( docValue )
	end
	
	return hasProxy
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasMasterOnOz( docValue )

	local master_href = docValue
				and docValue._serverProvidedMetadata
				and docValue._serverProvidedMetadata.links
				and docValue._serverProvidedMetadata.links[ "/rels/master" ]
				and docValue._serverProvidedMetadata.links[ "/rels/master" ].href
			
	if master_href then
		return true
	end
	
	return false
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasMasterLocally( docValue )

	local localPath = derefOpt( docValue, '_localOnly', 'originalFilePath' )

	if localPath then
		return true
	end

	localPath = derefOpt( docValue, '_localOnly', 'latestOriginal', 'filePath' )

	if localPath then
		return true
	end

	return false
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasMaster( docValue )

	local hasMaster = LrmAssetUtils.hasMasterOnOz( docValue )
	
	if not hasMaster then
		hasMaster = LrmAssetUtils.hasMasterLocally( docValue )
	end
	
	return hasMaster
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasPreviewLocally( docValue )

	local localOnly = docValue._localOnly or {}
	
	local hasPreview = derefOpt( localOnly, 'preview' )
	
	if hasPreview then
		return true
	else
		return false
	end
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasThumbnailLocally( docValue )

	local localOnly = docValue._localOnly or {}
	
	local hasThumbnail = derefOpt( localOnly, 'thumbnail2x' )
	
	if hasThumbnail then
		return true
	else
		return false
	end
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.isVideo( docValue )

	local video = docValue.type == 'asset' and docValue.subtype == 'video'
	
	if video then
		return true
	else
		return false
	end
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.isStack( docValue )

	return not not ( docValue.subtype == 'stack' )

end

--------------------------------------------------------------------------------

function LrmAssetUtils.getContentTypeNumber( docValue )

	local contentTypeNumber = 0
		
	local contentType = derefOpt( docValue, 'importSource', 'contentType' )
		or derefOpt( docValue, '_serverProvidedMetadata', 'links', '/rels/master', 'content_type' )
	if contentType then
		contentTypeNumber = WFOzContentTypes.getContentTypeNumberForContentType( contentType )
	else
		local subtype = derefOpt( docValue, '_serverProvidedMetadata', 'subtype' )
		contentTypeNumber = WFOzContentTypes.getContentTypeNumberRangeForSubType( subtype )
	end
	
	return contentTypeNumber
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.isVideoContentTypeNumber( contentTypeNumber )

	-- if we don't get a valid content type number we fall back to image
	
	if contentTypeNumber then
	
		local videoLow, videoHigh = WFOzContentTypes.getContentTypeNumberRangeForSubType( 'video' )
			
		if contentTypeNumber >= videoLow and contentTypeNumber <= videoHigh then
		
			return true
			
		end
		
	end
	
	return false

end

--------------------------------------------------------------------------------

function LrmAssetUtils.getXmp( assetInfo )

	local xmp = assetInfo.xmp or {}
	local copyright = KSTableUtils.derefOpt( xmp, "dc", "rights" )
	local exposureTime = KSTableUtils.derefOpt( xmp, "exif", "ExposureTime" )
	local fnumber = KSTableUtils.derefOpt( xmp, "exif", "FNumber" )
	local apertureValue = KSTableUtils.derefOpt( xmp, "exif", "ApertureValue" )
	local focalLength = KSTableUtils.derefOpt( xmp, "exif", "FocalLength" )
	local isoSpeedRatings = KSTableUtils.derefOpt( xmp, "exif", "ISOSpeedRatings" )
	local flashFired = KSTableUtils.derefOpt( xmp, "exif", "FlashFired" )
	local make = KSTableUtils.derefOpt( xmp, "tiff", "Make" )
	local model = KSTableUtils.derefOpt( xmp, "tiff", "Model" )
	local lens = KSTableUtils.derefOpt( xmp, "aux", "Lens" )
	
	-- exposure
	if exposureTime and type( exposureTime ) == 'table' and exposureTime[ 1 ] ~= nil then
		local expTime = exposureTime[ 1 ]
		if #exposureTime == 2 then
			local denominator = exposureTime[ 2 ]
			if denominator ~= 1 then
				if expTime == 1 then
					expTime = expTime .. "/" .. exposureTime[2]
				else
					if denominator ~= 0 then
						expTime = string.format ( "%.1f", expTime / denominator )
					end
				end
			end
		end
		exposureTime = tostring( expTime )
	else
		exposureTime = ""
	end
	
	-- f number
	if fnumber and type( fnumber ) == 'table' and fnumber[ 1 ] ~= nil then
		local denominator = 1
		if #fnumber == 2 and fnumber[ 2 ] ~= 0 then
			denominator = fnumber[ 2 ]
		end
		
		fnumber = string.format( "%.1f", fnumber[1] / denominator )
	else
		fnumber = ""
	end
	
	-- aperture
	if apertureValue and type( apertureValue ) == 'table' and apertureValue[ 1 ] ~= nil then
		local denominator = 1
		if #apertureValue == 2 and apertureValue[ 2 ] ~= 0 then
			denominator = apertureValue[ 2 ]
		end
		apertureValue = string.format( "%.1f", apertureValue[ 1 ] / denominator )
	else
		apertureValue = ""
	end
	
	-- focal length
	if focalLength and type( focalLength ) == 'table' and focalLength[ 1 ] ~= nil then
		local denominator = 1
		if #focalLength == 2 and focalLength[ 2 ] ~= 0 then
			denominator = focalLength[ 2 ]
		end
		local focalLengthValue = focalLength[1] / denominator;

		local _, fractionalPart = math.modf(focalLengthValue)
		if fractionalPart == 0 then
			focalLength = string.format( "%.0f",  focalLengthValue )
		else
			focalLength = string.format( "%.2f",  focalLengthValue )
		end
	else
		focalLength = ""
	end
	
	-- iso speed ratings
	local iso = ""
	if isoSpeedRatings and type( isoSpeedRatings ) == 'number' then
		iso = tostring( isoSpeedRatings )
	end

	-- flash fired
	local flash = false
	if flashFired and flashFired == true then
		flash = true
	end

	local xmpTable = {
		copyright = copyright,
		exposureTime = exposureTime,
		fnumber = fnumber,
		apertureValue = apertureValue,
		focalLength = focalLength,
		isoSpeedRatings = iso,
		flashFired = flash,
		cameraMake = make,
		cameraModel = model,
		make = make,
		model = model,
		lens = lens,
	}
	
	return xmpTable
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.getRenditionRevision( docValue, ozRenditionName )

	local localDescription = KSTableUtils.derefOpt( docValue, '_localOnly', ozRenditionName  )

	local localRenditionIsForRev = localDescription and localDescription.revId
	
	return localRenditionIsForRev
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.getLensProfileInfo( docValue )

	local info = KSTableUtils.derefOpt( docValue, 'develop', 'profiles', 'lens'  )
	if info and info.href and info.filename and string.sub( info.href, 1, 1 ) == "/" then
	
		local masterUrl = info.href
		return {
			filename = info.filename,
			href = masterUrl,
			uniqueId = info.href
		}
	end
	
	return nil
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.getCameraProfileInfo( docValue )

	local info = KSTableUtils.derefOpt( docValue, 'develop', 'profiles', 'camera'  )
	if info and info.href and info.filename and string.sub( info.href, 1, 1 ) == "/" then
	
		local masterUrl = info.href
		return {
			filename = info.filename,
			href = masterUrl,
			uniqueId = info.href
		}
	end
	
	return nil
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.isDeveloped( docValue )

	local xmpCameraRaw = KSTableUtils.derefOpt( docValue, 'develop', 'xmpCameraRaw' )
	
	if xmpCameraRaw and (type(xmpCameraRaw) == 'table' or xmpCameraRaw ~= "") then
		return true
	end
	
	return false
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasExternalXmpDevelop( docValue )

	local xmpCameraRaw = KSTableUtils.derefOpt( docValue, 'develop', 'xmpCameraRaw' )
	
	if xmpCameraRaw and type(xmpCameraRaw) == 'table' and xmpCameraRaw.sha256 then
		return true
	end
	
	return false
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.hasExternalXmpDevelopLocally( docValue )

	local xmpCameraRaw = KSTableUtils.derefOpt( docValue, 'develop', 'xmpCameraRaw' )

	local xmpLocalFiles = KSTableUtils.derefOpt( docValue, '_localOnly', 'files', 'xmp_develop' )
	
	if xmpLocalFiles and xmpLocalFiles.path and xmpLocalFiles.sha256 and xmpCameraRaw.sha256 
	  and xmpLocalFiles.sha256 == xmpCameraRaw.sha256 then
		
		if WFPlatformInfo.isAndroid then
			local documentsPath = KSPathUtils.getStandardFilePath( 'carouselDocuments' )

			if WFFileUtils.exists( KSPathUtils.child( documentsPath, xmpLocalFiles.path ) ) then
		return true
			end
		else
			return true
		end
	
	end
	
	return false
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.getDefaultAssetRating( docValue )

	local rating = 0
	local ratings = KSTableUtils.derefOpt( docValue, 'ratings' )
	if ratings and type( ratings ) == 'table' then
		for k, v in pairs( ratings ) do
			if v  and type( v ) == 'table' then
				if #k == 32 then
					if v.rating ~= nil then
						rating = v.rating
					end
					break
				end
			end
		end
	end
	
	return rating
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.getDefaultAssetReview( docValue )

	local flag = 'unflagged'
	local reviews = KSTableUtils.derefOpt( docValue, 'reviews' );
	if reviews and type( reviews ) == 'table' then
		for k, v in pairs( reviews ) do
			if v  and type( v ) == 'table' then
				if #k == 32 then
					if v.flag ~= nil then
						flag = v.flag
					end
					break
				end
			end
		end
	end
	
	return flag
	
end

--------------------------------------------------------------------------------

function LrmAssetUtils.getCapturedWithAdobeCamera( docValue )

	local isCapturedWithAdobeCamera = derefOpt( docValue, 'xmp', 'xmp', 'CreatorTool' )

	isCapturedWithAdobeCamera = isCapturedWithAdobeCamera and type( isCapturedWithAdobeCamera ) == 'string' and
		( string.find( isCapturedWithAdobeCamera, 'Adobe Camera' ) == 1 or
			string.find( isCapturedWithAdobeCamera, 'Thio' ) == 1 or
			string.find( isCapturedWithAdobeCamera, 'Lightroom CC' ) == 1 )
		or false

		return isCapturedWithAdobeCamera
end

--------------------------------------------------------------------------------

local function isLatestMetadataUploaded( docValue )

	local spm = docValue and docValue._serverProvidedMetadata

	local links = KSTableUtils.derefOpt( spm, 'links' )

	-- Do we have nothing on Oz yet?

	if not links then
		return false
	end

	if spm then

		local revId = docValue._revId

		local revIds = spm.revision_ids

		if revIds then

			local cnt = #revIds

			if cnt == 0 then
				-- SPAM [matthewj 2016-09-02]: if the revision_ids table is present but
				-- empty this is likely an older non-asset type doc (e.g. album) from
				-- Oz which did not traditionally have revision ids. We will consider
				-- these as synced for UI purposes as in most cases it will be. However,
				-- the first local change to the doc will not trigger the UI status to
				-- move to 'syncing' as the revision id table will still be empty. Once
				-- this actually syncs to Oz and back the revision_ids array will no
				-- longer be empty and all subsequent changes should correctly reflect
				-- in changes to the sync status for the UI
				return true
			end

			-- if local _revId can be found in revision_ids array tracked by Oz
			-- the latest metadata is uploaded

			for i = cnt, 1, -1 do

				local ozRevId = revIds[ i ]
				if revId == ozRevId then
					return true
				end

			end

		end

	end

	return false

end

--------------------------------------------------------------------------------

local function doesAssetNeedMasterUploaded( docValue )

	local hasMasterUrl = LrmAssetUtils.hasMasterOnOz( docValue )
	local hasLocalOriginal = LrmAssetUtils.hasMasterLocally( docValue )
	local isOzOriginalInvalid = not not derefOpt( docValue, '_serverProvidedMetadata', 'links', '/rels/master', 'invalid' )

	local needsUpload = false

	if hasLocalOriginal and not hasMasterUrl then

		needsUpload = true

	elseif hasLocalOriginal and hasMasterUrl and isOzOriginalInvalid then

		needsUpload = true

	end

	return needsUpload

end

--------------------------------------------------------------------------------

local function doesAssetNeedDevelopXmpUploaded( docValue )

	local hasXmpDevelopUrl = not not derefOpt( docValue, '_serverProvidedMetadata', 'links', '/rels/xmp/develop', 'href' )
	local hasDevelopXmpLocally = LrmAssetUtils.hasExternalXmpDevelopLocally( docValue )
	local isXmpDevelopInvalid = not not derefOpt( docValue, '_serverProvidedMetadata', 'links', '/rels/xmp/develop', 'invalid' )

	local needsUpload = false

	if  hasDevelopXmpLocally and not hasXmpDevelopUrl then

		needsUpload = true

	elseif hasDevelopXmpLocally and hasXmpDevelopUrl and isXmpDevelopInvalid then

		needsUpload = true

	end

	return needsUpload

end

--------------------------------------------------------------------------------

local function doesAssetNeedProxyOrMasterDownloaded( docValue )

	local hasLocalOriginal = LrmAssetUtils.hasMasterLocally( docValue )
	local hasLocalProxy = LrmAssetUtils.hasPreviewLocally( docValue )

	local needsDownload = true

	if hasLocalOriginal or hasLocalProxy then
		needsDownload = false
	end

	return needsDownload

end

--------------------------------------------------------------------------------

local function doesAssetNeedDevelopXmpDownloaded( docValue )

	local needsDownload = false

	if LrmAssetUtils.hasExternalXmpDevelop( docValue )
				and not LrmAssetUtils.hasExternalXmpDevelopLocally( docValue ) then

		return true

	end

	return needsDownload

end

--------------------------------------------------------------------------------


function LrmAssetUtils.getSyncStatus( docValue )

	-- regular states
	local kAssetNeedsMetadataUpload 			= 'needsMetadataUpload'
	local kAssetNeedsMasterUpload 				= 'needsMasterUpload'
	local kAssetNeedsDevelopXmpUpload			= 'needsDevelopXmpUpload'
	local kAssetNeedsProxyOrMasterDownload		= 'needsProxyOrMaster'
	local kAssetNeedsDevelopXmpDownload			= 'needsDevelopXmp'
	local kAssetComplete 						= 'complete'
	local kAssetCompleteNoMaster 				= 'completeNoMaster'
	local kAssetStackComplete 					= 'stackComplete'
	-- error states
	local kAssetStorageLimitExceeded			= 'storageLimitExceeded'
	local kAssetTrialOrSubscriptionExpired		= 'trialOrSubscriptionExpired'
	local kAssetMultipartCompleteError			= 'multipartCompleteError'

	local syncStatus = kAssetNeedsMetadataUpload

	if LrmAssetUtils.isStack( docValue ) then

		if not isLatestMetadataUploaded( docValue ) then
			syncStatus = kAssetNeedsMetadataUpload
		else
			syncStatus = kAssetStackComplete
		end

	else

		if not isLatestMetadataUploaded( docValue ) then

			if WFCatOzErrors.hasStorageExceededError( docValue ) then
				syncStatus = kAssetStorageLimitExceeded
			elseif WFCatOzErrors.hasTrialOrSubscriptionExpiredError( docValue ) then
				syncStatus = kAssetTrialOrSubscriptionExpired
			elseif WFCatOzErrors.hasMultipartCompleteError( docValue ) then
				syncStatus = kAssetMultipartCompleteError
			else
				syncStatus = kAssetNeedsMetadataUpload
			end

		elseif doesAssetNeedMasterUploaded( docValue ) then

			if WFCatOzErrors.hasStorageExceededError( docValue ) then
				syncStatus = kAssetStorageLimitExceeded
			elseif WFCatOzErrors.hasTrialOrSubscriptionExpiredError( docValue ) then
				syncStatus = kAssetTrialOrSubscriptionExpired
			elseif WFCatOzErrors.hasMultipartCompleteError( docValue ) then
				syncStatus = kAssetMultipartCompleteError
			else
				syncStatus = kAssetNeedsMasterUpload
			end

		elseif doesAssetNeedDevelopXmpUploaded( docValue ) then

			if WFCatOzErrors.hasStorageExceededError( docValue ) then
				syncStatus = kAssetStorageLimitExceeded
			elseif WFCatOzErrors.hasTrialOrSubscriptionExpiredError( docValue ) then
				syncStatus = kAssetTrialOrSubscriptionExpired
			elseif WFCatOzErrors.hasMultipartCompleteError( docValue ) then
				syncStatus = kAssetMultipartCompleteError
			else
				syncStatus = kAssetNeedsDevelopXmpUpload
			end

		elseif doesAssetNeedProxyOrMasterDownloaded( docValue ) then

			syncStatus = kAssetNeedsProxyOrMasterDownload

		elseif doesAssetNeedDevelopXmpDownloaded( docValue ) then

			syncStatus = kAssetNeedsDevelopXmpDownload

		else

			if LrmAssetUtils.hasMasterLocally( docValue ) then

				syncStatus = kAssetComplete

			else

				syncStatus = kAssetCompleteNoMaster

			end

		end

	end

	return syncStatus

end

--------------------------------------------------------------------------------

function LrmAssetUtils.getImportSourceCameraAndLensName( docValue )

	-- These values will be empty for old assets. We might want to default them here.

	local importSourceCameraName = derefOpt( docValue, 'importSource', 'camera', 'name' )
	local importSourceLensName = derefOpt( docValue, 'importSource', 'lens', 'name' )
	return importSourceCameraName, importSourceLensName

end

--------------------------------------------------------------------------------

function LrmAssetUtils.preferableBinaryForDevelop( docValue, binaryPreference )

	WFArguments.check( 'table', 'binaryStrategy' )
	
	local hasProxy = LrmAssetUtils.hasProxy( docValue )
	local hasMaster = LrmAssetUtils.hasMaster( docValue )
	
	local localUrl = KSTableUtils.derefOpt( docValue, '_localOnly', 'localUrl' )
	local hasProxyLocally = LrmAssetUtils.hasProxyLocally( docValue )
	local hasMasterLocally = LrmAssetUtils.hasMasterLocally( docValue )
	
	local hasLocalBinary = hasProxyLocally or hasMasterLocally
	
	local useBinaryFromCameraRoll = localUrl and not hasLocalBinary
	
	if useBinaryFromCameraRoll then
		return LrmConfig.StrategyPreferMaster
	end
	
	if ( hasProxy and binaryPreference == LrmConfig.StrategyPreferProxy ) or 
		( binaryPreference == LrmConfig.StrategyPreferMaster and not hasMaster ) then
		
		return LrmConfig.StrategyPreferProxy
		
	else
	
		return LrmConfig.StrategyPreferMaster
		
	end
	
end

--------------------------------------------------------------------------------

return LrmAssetUtils
