--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.reactive
local WFRxRunUntilDone = require 'WFRxRunUntilDone'

	-- WF trash
local WFCatTrash = require 'WFCatTrash'

	-- WF.core.logger
local WFLogger = require 'WFLogger'

local WFCapabilities = require 'WFCapabilities'

--==============================================================================

local runUntilDone = WFRxRunUntilDone

local logger = WFLogger( 'LrmLibraryDelete' )

--==============================================================================

local LrmLibraryDelete = {}

--==============================================================================

local function getPusherControlFuncs( catalog )
	return {
		pauseReturningCounter = function()
			return catalog:getSyncClient():pausePusherReturningCounter()
		end,
		resume = function()
			catalog:getSyncClient():resumePusher()
				:call( runUntilDone )
		end,
	}
end

--------------------------------------------------------------------------------

function LrmLibraryDelete.deleteAssetsModel( self, assetIds )

	local result

	local enableCloudTrash = WFCapabilities.getCapability( "WFCatDelete.enableCloudTrash" ) and true or false

	if enableCloudTrash then
		result = self._wfCatalogStream
			:tap( function( wfCatalog )
				local pusherControl = getPusherControlFuncs( self.catalog )
				WFCatTrash.moveAssetsToTrash( wfCatalog, assetIds, pusherControl )
					:call( runUntilDone )
			end )
			:call( runUntilDone )
			:map( function()
				return true
			end )
			
	else
		
		local overlays = {}
		
		for i, assetId in ipairs( assetIds ) do

			overlays[ i ] = {
				_id = assetId,
				_deleted = true
			}

		end
		
		result = self.catalog:docStore():bulkApplyOverlay( overlays )
			:call( runUntilDone )
			
	end
	
	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryDelete.restoreTrashAssetsModel( self, trashAssetIds )

	local result = self._wfCatalogStream
		:tap( function( wfCatalog )
		
			local pusherControl = getPusherControlFuncs( self.catalog )
			WFCatTrash.restoreTrashAssets( wfCatalog, trashAssetIds, pusherControl )
				:call( runUntilDone )
				
		end )
		:call( runUntilDone )
		:map( function()
			return true
		end )

	return {
		value = result
	}
end

--------------------------------------------------------------------------------

function LrmLibraryDelete.deleteTrashAssetsModel( self, trashAssetIds )

	local result = self._wfCatalogStream
		:tap( function( wfCatalog )
		
			local pusherControl = getPusherControlFuncs( self.catalog )
			WFCatTrash.deleteTrashAssets( wfCatalog, trashAssetIds, pusherControl )
				:call( runUntilDone )
				
		end )
		:call( runUntilDone )
		:map( function()
			return true
		end )
	
	return {
		value = result
	}
end

--------------------------------------------------------------------------------

return LrmLibraryDelete
