--[[----------------------------------------------------------------------------

WFCatTrash.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2019 Adobe
All Rights Reserved.

NOTICE: All information contained herein is, and remains
the property of Adobe and its suppliers, if any. The intellectual
and technical concepts contained herein are proprietary to Adobe
and its suppliers and are protected by all applicable intellectual
property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material
is strictly forbidden unless prior written permission is obtained
from Adobe.

----------------------------------------------------------------------------]]--

local KSTableUtils = require 'KSTableUtils'

--==============================================================================

local WFCatTrashUtils = {}

--------------------------------------------------------------------------------

local derefOpt = KSTableUtils.derefOpt

--------------------------------------------------------------------------------

function WFCatTrashUtils.createFromInfo( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getCreateFromId( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'id' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getCreateWithRevisionId( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createWithRevId' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.originalExistsOnOz( docValue )
	-- Return true if the asset this docValue was copied from existed on Oz.
	-- This works for both regular and trash documents.
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom' ) ~= nil
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getMasterIsUploaded( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'uploaded', '/rels/master' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getProxyIsUploaded( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'uploaded', '/rels/proxy_type/2560' )
		or derefOpt( docValue, '_localOnly', 'universal', 'trash', 'uploaded', '/rels/proxy_type/panorama' )
		or derefOpt( docValue, '_localOnly', 'universal', 'trash', 'uploaded', '/rels/proxy_type/720p' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getXmpDevelopIsUploaded( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'uploaded', '/rels/xmp/develop' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getOriginalAssetMasterLink( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'links', '/rels/master' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getOriginalAssetXmpDevelopLink( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'links', '/rels/xmp/develop' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getOriginalAssetXmpLink( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'links', '/rels/xmp' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getOriginalAssetLinkForKey( docValue, linksKey )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'links', linksKey )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getOriginalAssetUrlForLinkKey( docValue, linksKey )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'links', linksKey, 'href' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.getOriginalAssetLinks( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'createFrom', 'links' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.isDeleteTrash( docValue )
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'isDeleteTrash' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.trashAssetInfo( docValue )
-- NOTE: We could remove this as it's no longer used
-- by WFCatTrash or WFCatMetadataPusher but the mobile purgeAgents are
-- currently using it.

	-- This info is attached to a doc of type 'asset' with subtype = 'image' or
	-- 'video' that has been deleted.
	-- This is probably not needed anymore but it may be helpful in understanding
	-- what happened to the asset.
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'trash_asset' )
end

--------------------------------------------------------------------------------

function WFCatTrashUtils.restoredToInfo( docValue )
-- NOTE: We could remove this as it's no longer used
-- by WFCatTrash or WFCatMetadataPusher but the mobile purgeAgents are
-- currently using it.

	-- This info is attached to a doc of type 'asset' with subtype = 'deleted_image'
	-- or 'deleted_video' that has been deleted.
	-- This is probably not needed anymore but it may be helpful in understanding
	-- what happened to the asset.
	return derefOpt( docValue, '_localOnly', 'universal', 'trash', 'restoredTo' )
end

--==============================================================================

return WFCatTrashUtils
