--[[----------------------------------------------------------------------------

WFCodecUsage.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- Rx
local Rx = require 'Rx'
local WFRxScheduler = require 'WFRxScheduler'

	-- WF.core
local WFArguments = require 'WFArguments'
local WFLogger = require 'WFLogger'

	-- WF.acr.wrapper
local WFCrInfo = require 'WFCrInfo'
local WFCodecUsageImp = require 'WFCodecUsageImp'

--------------------------------------------------------------------------------

local logger = WFLogger 'WFCodecUsage'
local check = WFArguments.check

local Observable = Rx.Observable
local kPollingDelay = 5

--------------------------------------------------------------------------------

local function getLibde265UsageStream()
	return Observable.timer( kPollingDelay, kPollingDelay, WFRxScheduler.sharedScheduler() )
		:flatMapLatest( function()
			local libde265Used = WFCodecUsageImp.isLibde265Used()
			
			if libde265Used then
				logger:infof( 'Detected libde265 library usage' )
				return Observable.just( libde265Used )
			else
				return Observable.empty()
			end
		end )
		:finally( function()
			logger:infof( 'Shutting down' )
		end )
end

--------------------------------------------------------------------------------

local function getCodecUsageStream( entitlement )
	check( 'table' )
	
	-- Only one for now
	WFCrInfo.setHeicEntitlement( entitlement.heic )
	return getLibde265UsageStream()
end

--------------------------------------------------------------------------------

return {
	getCodecUsageStream = getCodecUsageStream,
}
